use mongodb::Database;
use mongodb::bson::oid::ObjectId;
use mongodb::bson::{doc, Document, Bson};
use futures::stream::{TryStreamExt, StreamExt};
use mongodb::options::FindOptions;
use std::sync::Arc;
use crate::models::administrator::*;
use crate::storages::traits::AdministratorRepository;
use async_trait::async_trait;


pub struct MongoAdministratorRepository {
    pub database: Arc<Database>
}

#[async_trait]
impl AdministratorRepository for MongoAdministratorRepository {
    async fn read(&self, id: String) -> Result<Administrator, String> {
        let collection = self.database.collection::<Document>("administrators");
        let filter = doc! {
            "_id": ObjectId::parse_str(&id).unwrap()
        };
        let result = collection.find_one(filter, None).await;
        match result {
            Ok(Some(document)) => {
                let id = document.get_object_id("_id").unwrap().to_hex();
                let username = document.get_str("username").unwrap().to_string();
                let password = document.get_str("password").unwrap().to_string();
                let name = document.get_str("name").unwrap().to_string();
                let middlename = document.get_str("middlename").unwrap().to_string();
                let surname = document.get_str("surname").unwrap().to_string();
                let car_station_id = document.get_object_id("car_station_id").unwrap().to_hex();
                let administrator = Administrator {id, username, password, name, surname, middlename, car_station_id};
                Ok(administrator)
            },
            Ok(None) => Err("Administrator not found".to_string()),
            Err(e) => Err(e.to_string())
        }
    }

    async fn find_by_username(&self, username: &str) -> Result<Administrator, String> {
        let collection = self.database.collection::<Document>("administrators");
        let filter = doc! {
            "username": username.to_string()
        };
        let result = collection.find_one(filter, None).await;
        match result {
            Ok(Some(document)) => {
                let id = document.get_object_id("_id").unwrap().to_hex();
                let username = document.get_str("username").unwrap().to_string();
                let password = document.get_str("password").unwrap().to_string();
                let name = document.get_str("name").unwrap().to_string();
                let middlename = document.get_str("middlename").unwrap().to_string();
                let surname = document.get_str("surname").unwrap().to_string();
                let car_station_id = document.get_object_id("car_station_id").unwrap().to_hex();
                let administrator = Administrator {id, username, password, name, surname, middlename, car_station_id};
                Ok(administrator)
            },
            Ok(None) => Err("Administrator not found".to_string()),
            Err(e) => Err(e.to_string())
        }
    }
}