use crate::models::administrator::Administrator;
use crate::models::client::*;
use crate::models::car_station::*;
use crate::models::car::*;
use crate::models::rent::*;
use async_trait::async_trait;

#[async_trait]
pub trait ClientRepository {
    async fn create(&self, client: BindingClient) -> Result<Client, String>;
    async fn read(&self, id: String) -> Result<Client, String>;
    async fn read_all(&self) -> Result<Vec<Client>, String>;
    async fn update(&self, id: String, client: BindingClient) -> Result<Client, String>;
    async fn delete(&self, id: String) -> Result<(), String>;
}

#[async_trait]
pub trait CarRepository {
    async fn create(&self, car: BindingCar) -> Result<Car, String>;
    async fn read(&self, id: String) -> Result<Car, String>;
    async fn read_all(&self) -> Result<Vec<Car>, String>;
    async fn update(&self, id: String, car: BindingCar) -> Result<Car, String>;
    async fn delete(&self, id: String) -> Result<(), String>;
    async fn get_report(&self) -> Result<Vec<Report>, String>;
}

#[async_trait]
pub trait CarStationRepository {
    async fn create(&self, car_station: BindingCarStation) -> Result<CarStation, String>;
    async fn read(&self, id: String) -> Result<CarStation, String>;
    async fn read_all(&self) -> Result<Vec<CarStation>, String>;
    async fn update(&self, id: String, car_station: BindingCarStation) -> Result<CarStation, String>;
    async fn delete(&self, id: String) -> Result<(), String>;
}

#[async_trait]
pub trait RentRepository {
    async fn create(&self, rent: BindingRent) -> Result<Rent, String>;
    async fn read(&self, id: String) -> Result<Rent, String>;
    async fn read_all(&self) -> Result<Vec<Rent>, String>;
    async fn update(&self, id: String, rent: BindingRent) -> Result<Rent, String>;
    async fn delete(&self, id: String) -> Result<(), String>;
    async fn end_rent(&self, id: String) -> Result<(), String>;
}

#[async_trait]
pub trait OwnerRepository {
    async fn create(&self, client: BindingClient) -> Result<Client, String>;
    async fn read(&self, id: String) -> Result<Client, String>;
    async fn read_all(&self) -> Result<Vec<Client>, String>;
    async fn update(&self, id: String, client: BindingClient) -> Result<Client, String>;
    async fn delete(&self, id: String) -> Result<(), String>;
}

#[async_trait]
pub trait AdministratorRepository {
    async fn read(&self, id: String) -> Result<Administrator, String>;
    async fn find_by_username(&self, username: &str) -> Result<Administrator, String>;
}

#[async_trait]
pub trait BenchmarkRepository {
    async fn benchmark(&self) -> Result<i64, String>;
}